function Game() {
    this.$recordsPopup = $("#records_popup");
    this.$popupPointsParagraph = $("#points_paragraph");
    this.$popupPlayAgainBtn = $("#records_popup_btn");
    this.$popupPlayAgainBtn.click(function(e) {
        objects.game.$recordsPopup.addClass(domElements.invisibleClassName);
        objects.game.startNewGame();
        utils.turnOnScreen(domElements.$gameScreen);
    });

    this.$pointsSpan = $("#points_span");
    this.$livesSpan = $("#lives_span");
    this.lives = this.points = this.levelSpeed = undefined;
    this.player = undefined;
    this.priseChance = this.platformChance = undefined;
    this.prises = this.platforms = undefined;

    this.tick = this.timer = this.clearTimeout = this.pauseTimeout = undefined;
    this.background = this.sideBricks = undefined;

    this.setView();
}

Game.prototype.setView = function() {
    this.background = new paper.Path.Rectangle(new paper.Point(0, 0), new paper.Size(constants.GAME_WIDTH, constants.GAME_HEIGHT));

    this.sideBricks = new SideBricks();
    this.player = new Player();
}

Game.prototype.startNewGame = function() {
    this.prises = new Array();
    this.platforms = new Array();
    this.priseChance = settings.START_PRISE_CHANCE_PER_SECOND;
    this.platformChance = settings.START_PLATFORM_CHANCE;
    this.lives = settings.START_LIVES;
    this.points = 0;
    this.setTexts();
    this.levelSpeed = settings.START_LEVEL_SPEED;
    this.tick = 0;
    this.sideBricks.setToStart();
    this.player.setToStart();
    this.timer = window.setInterval(function() {
        objects.game.onFrame();
    }, constants.MILISECONDS_FOR_FRAME);
}

Game.prototype.pauseGame = function(pauseMilliseconds) {
    if(typeof (this.timer) == "undefined") return;
    window.clearInterval(this.timer);
    this.timer = undefined;
    this.pauseTimeout = window.setTimeout(function(){objects.game.resumeGame();}, pauseMilliseconds);
}

Game.prototype.resumeGame = function() {
    window.clearTimeout(this.pauseTimeout);
    this.pauseTimeout = undefined;
    this.timer = window.setInterval(function() {objects.game.onFrame();}, constants.MILISECONDS_FOR_FRAME);
}

Game.prototype.onFrame = function() {
    this.tick += constants.MILISECONDS_FOR_FRAME;
    if(this.tick % (settings.LEVEL_DURATION_IN_SECONDS * 1000) == 0) {
        this.levelSpeed += settings.PLUS_LEVEL_SPEED;
        this.priseChance += settings.PLUS_LEVEL_PRISE_CHANCE;
        this.priseChance = Math.min(this.priseChance, settings.MAX_PRISE_CHANCE);
        this.platformChance -= settings.MIN_LEVEL_PLATFORM_CHANCE;
        this.platformChance = Math.max(settings.MIN_LEVEL_PLATFORM_CHANCE, this.platformChance);
    }
    if(typeof (variables.pageX) != "undefined") this.setNewMovementPoint();

    if(this.tick % 1000 == 0) {
        this.spawnElements();
    }
    this.sideBricks.onFrame();
    this.player.onFrame();
    this.setElementsMovement();

	paper.view.draw();
}

Game.prototype.spawnElements = function() {
    if(Math.random() < this.priseChance / 100) {
        var prise = new Prise();
        this.prises.push(prise);
    }
    if(Math.random() < this.platformChance / 100) {
        var platform = new Platform();
        this.platforms.push(platform);
    }
}

Game.prototype.setElementsMovement = function() {
    var i = 0;
    while(i < this.prises.length) {
        this.prises[i].onFrame();
        if(this.prises[i].img.position.y < 0 - this.prises[i].height / 2) {
            this.prises[i].clear();
            this.prises.splice(i, 1);
            continue;
        }

        if(this.player.img.position.getDistance(this.prises[i].img.position) < settings.PLAYER_HEIGHT / 2 + this.prises[i].height / 2) {
            this.points += this.prises[i].points;
            this.setTexts();
            this.prises[i].clear();
            this.prises.splice(i, 1);
            continue;
        }

        i++;
    }

    i = 0;
    while(i < this.platforms.length) {
        this.platforms[i].onFrame();
        if(this.platforms[i].img.position.y < 0 - 100) {
            this.platforms[i].clear();
            this.platforms.splice(i, 1);
            continue;
        }
        if(typeof (this.platforms[i].opacityCount) != "undefined" && this.platforms[i].opacityCount <= 0) {
            if(this.platforms[i] == this.player.activePlatform) this.player.activePlatform = undefined;
            this.platforms[i].clear();
            this.platforms.splice(i, 1);
            continue;
        }
        if(this.platforms[i].img.position.y < this.player.img.position.y || this.platforms[i].img.position.y
                - this.player.img.position.y > settings.PLAYER_HEIGHT / 2 + settings.PLATFORM_BLOCK_SIZE / 2) {
            i++;
            continue;
        }
        if(this.player.img.position.x < this.platforms[i].img.position.x - this.platforms[i].width / 2 - this.player.img.size.width / 4
                || this.player.img.position.x > this.platforms[i].img.position.x + this.platforms[i].width / 2 + this.player.img.size.width / 4) {
            i++;
            continue;
        }

        if(typeof (this.player.activePlatform) != "undefined" && this.player.activePlatform == this.platforms[i]) {
            i++;
            continue;
        }

        if(this.platforms[i].type == settings.NEEDLE_PLATFORM_TYPE && typeof (this.player.invulnarableTimer) == "undefined") {
            this.setMinusOneLife();
            this.platforms[i].clear();
            this.platforms.splice(i, 1);
            continue;
        }

        if(typeof (this.player.invulnarableTimer) == "undefined") {
            this.player.activePlatform = this.platforms[i];
            this.player.img.position.y = this.platforms[i].img.position.y - settings.PLATFORM_BLOCK_SIZE / 2 - settings.PLAYER_HEIGHT / 2;
            if(typeof (this.platforms[i].opacityCount) == "undefined" && this.platforms[i].type == settings.SOFT_PLATFORM_TYPE)
                this.platforms[i].opacityCount = 1;
        }
        i++;
    }
}

Game.prototype.setNewMovementPoint = function() {
    var movementPoint = utils.getCanvasPoint(variables.pageX, variables.pageY);
    this.player.setNewPoint(movementPoint);
    variables.pageX = variables.pageY = undefined;
}

Game.prototype.endGame = function() {
    window.clearInterval(this.timer);
    this.timer = undefined;
    this.clearTimer = window.setTimeout(function() {
        objects.game.removeGameObjects();
    }, constants.MILISECONDS_FOR_FRAME * 2);
}

Game.prototype.removeGameObjects = function() {
    window.clearTimeout(this.clearTimer);
    this.clearTimer = undefined;

    variables.currentScore = this.points;

    while(this.prises.length > 0) {
        this.prises[0].clear();
        this.prises.splice(0, 1);
    }
    while(this.platforms.length > 0) {
        this.platforms[0].clear();
        this.platforms.splice(0, 1);
    }

    if(settings.RECORDS_ENABLED) {
        objects.records.setNewRecords();
        utils.turnOnScreen(domElements.$recordScreen);
    }else {
        this.$recordsPopup.removeClass(domElements.invisibleClassName);
        this.$popupPointsParagraph.text(this.points);
    }
}

Game.prototype.setMinusOneLife = function() {
    this.lives--;
    this.setTexts();
    if(this.lives <= 0) this.endGame();
    this.player.setToStart();
}

Game.prototype.setTexts = function() {
    this.$pointsSpan.text(this.points);
    this.$livesSpan.text(this.lives);
}