function Player() {
    this.distanceSign = this.targetX = this.ySpeed = this.xSpeed = this.img = this.invulnarableTimer = undefined;
    this.activePlatform = undefined;
    this.setView();
}

Player.prototype.setView = function() {
    this.img = new paper.Raster(variables.playerImage);
    this.img.size.height = settings.PLAYER_HEIGHT;
    this.img.size.width = variables.playerImage.width * (settings.PLAYER_HEIGHT / variables.playerImage.height);
}

Player.prototype.setToStart = function() {
    this.targetX = undefined;
    this.activePlatform = undefined;
    this.img.position.y = settings.PLAYER_HEIGHT / 2;
    this.img.position.x = constants.GAME_WIDTH / 2;
    this.ySpeed = settings.PLAYER_FALLDOWN_SPEED;
    this.invulnarableTimer = 0;
}

Player.prototype.onFrame = function() {
    if(typeof (this.targetX) != "undefined") this.setXMovement();
    if(typeof (this.invulnarableTimer) != "undefined") {
        this.setInvulnerableTimer();
        return;
    }

    if(this.ySpeed < settings.PLAYER_FALLDOWN_SPEED) {
        this.ySpeed += settings.JUMP_GRAVITY;
    }

    if(typeof (this.activePlatform) == "undefined") {
        this.img.position.y += this.ySpeed;
    }else {
        this.img.position.y -= objects.game.levelSpeed;
        if(this.img.position.x < this.activePlatform.img.position.x - this.activePlatform.width / 2 - this.img.size.width / 4
                || this.img.position.x > this.activePlatform.img.position.x + this.activePlatform.width / 2 + this.img.size.width / 4) {
            this.activePlatform = undefined;
        }
    }
    if(this.img.position.y >= constants.GAME_HEIGHT + settings.PLAYER_HEIGHT / 2 || this.img.position.y <= 0 - settings.PLAYER_HEIGHT) {
        objects.game.setMinusOneLife();
    }
}

Player.prototype.setXMovement = function() {
    var sign = this.img.position.x > this.targetX;
    if(Math.abs(this.img.position.x - this.targetX) < settings.SIDE_SPEED / 2 || sign != this.distanceSign) {
        this.img.position.x = this.targetX;
        this.targetX = undefined;
        this.xSpeed = undefined;
        return;
    }

    if(Math.abs(this.img.position.x - this.targetX) < settings.SIDE_SPEED * 3) this.xSpeed /= 1.25;
    this.img.position.x = this.targetX > this.img.position.x ? this.img.position.x + this.xSpeed
            : this.img.position.x - this.xSpeed;
}

Player.prototype.setInvulnerableTimer = function() {
    this.invulnarableTimer += settings.MILISECONDS_FOR_FRAME;
    if(this.invulnarableTimer % 80 == 0) this.img.visible = !this.img.visible;
    if(this.invulnarableTimer > settings.INVULNERABILITY_MILISECONDS) {
        this.img.visible = true;
        this.invulnarableTimer = undefined;
    }
}

Player.prototype.setNewPoint = function(point) {
    if(point.y < this.img.position.y && typeof (this.activePlatform) != "undefined") {
        this.activePlatform = undefined;
        this.ySpeed = settings.PLAYER_FALLDOWN_SPEED * -1.25 - objects.game.levelSpeed;
    }
    if((typeof(this.targetX) == "undefined" || point.x != this.targetX) && point.x > settings.SIDE_BRICK_SIZE + this.img.size.width / 2
            && point.x < constants.GAME_WIDTH - settings.SIDE_BRICK_SIZE - this.img.size.width / 2) {
        this.targetX = point.x;
        this.distanceSign = this.img.position.x > this.targetX;
        this.xSpeed = settings.SIDE_SPEED;
    }
}